//
//  UINavigationBar+TitleImage.m
//  Demo
//
//  Created by zhenby on 13-8-16.
//  Copyright (c) 2013年 zhenby. All rights reserved.
//

#import "UINavigationBar+TitleImage.h"

#define IMAGE_DEFAULT_WIDTH         24.0
#define IMAGE_DEFAULT_HEIGHT        24.0
#define IMAGE_VIEW_TAG              1
#define TITLE_LABEL_TAG             2
#define IMAGE_AND_LABEL_SPACE       3.0

@implementation UINavigationBar (TitleImage)

#pragma mark - Category Methods

- (void)setTitleImage:(UIImage *)titleImage
{
    [self setTitleImage:titleImage imageSize:CGSizeMake(IMAGE_DEFAULT_WIDTH, IMAGE_DEFAULT_HEIGHT)];
}

- (void)setTitleImage:(UIImage *)titleImage imageSize:(CGSize)imageSize
{
    [self setTitle:nil titleImage:titleImage imageSize:imageSize];
}

- (void)setTitle:(NSString *)title titleImage:(UIImage *)titleImage
{
    [self setTitle:title
        titleImage:titleImage
         imageSize:CGSizeMake(IMAGE_DEFAULT_WIDTH, IMAGE_DEFAULT_HEIGHT)];
}

- (void)setTitle:(NSString *)title titleImage:(UIImage *)titleImage imageSize:(CGSize)imageSize
{
    [self addTitleLabelWithTitle:title];
    [self addTitleImageViewWithImage:titleImage imageSize:imageSize];
}


#pragma mark - Private methods

- (void)addTitleImageViewWithImage:(UIImage *)titleImage imageSize:(CGSize)imageSize
{
    [[self viewWithTag:IMAGE_VIEW_TAG] removeFromSuperview];
    
    if (titleImage)
    {
        CGFloat imageHeight = imageSize.height;
        CGFloat imageWidth = imageSize.width;
        CGRect imageRect = CGRectMake((self.bounds.size.width - imageWidth) / 2,
                                      (self.bounds.size.height - imageHeight) / 2,
                                      imageWidth, imageHeight);
        UIView *label = [self viewWithTag:TITLE_LABEL_TAG];
        CGSize titleSize = label.frame.size;
        if (label && titleSize.width > 0 && titleSize.height > 0)
        {
            UIView *label = [self viewWithTag:TITLE_LABEL_TAG];
            CGRect labelRect = label.frame;
            
            CGFloat imageX = (self.bounds.size.width - titleSize.width - imageWidth - IMAGE_AND_LABEL_SPACE) / 2;
            CGFloat imageY = (self.bounds.size.height - imageHeight) / 2;
            imageRect.origin = CGPointMake(imageX, imageY);
            
            // Reset Label Frame
            CGFloat labelX = imageX + imageWidth + IMAGE_AND_LABEL_SPACE;
            CGFloat labelY = labelRect.origin.y;
            labelRect.origin = CGPointMake(labelX, labelY);
            label.frame = labelRect;
        }
        UIImageView *imageView = [[UIImageView alloc] initWithImage:titleImage];
        imageView.frame = imageRect;
        imageView.tag = IMAGE_VIEW_TAG;
        [self addSubview:imageView];
    }
}

- (void)addTitleLabelWithTitle:(NSString *)title
{
    [[self viewWithTag:TITLE_LABEL_TAG] removeFromSuperview];
    
    if (title.length > 0)
    {
        UIFont *titleFont = [self.titleTextAttributes objectForKey:UITextAttributeFont];
        if (!titleFont)
        {
            titleFont = [UIFont boldSystemFontOfSize:20.0];
        }
        CGSize titleSize = [title sizeWithFont:titleFont constrainedToSize:self.bounds.size];
        CGFloat x = (self.bounds.size.width - titleSize.width) / 2;
        CGFloat y = (self.bounds.size.height - titleSize.height) / 2;
        UILabel *titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(x, y,
                                                                        titleSize.width,
                                                                        titleSize.height)];
        titleLabel.text = title;
        titleLabel.backgroundColor = [UIColor clearColor];
        titleLabel.font = titleFont;
        titleLabel.tag = TITLE_LABEL_TAG;
        titleLabel.textColor = [UIColor whiteColor];
        titleLabel.shadowColor = [UIColor grayColor];
        titleLabel.shadowOffset = CGSizeMake(-1.0, 0.0);
        
        UIColor *titleColor = [self.titleTextAttributes objectForKey:UITextAttributeTextColor];
        if (titleColor)
        {
            titleLabel.textColor = titleColor;
        }
        
        UIColor *shadowColor = [self.titleTextAttributes objectForKey:UITextAttributeTextShadowColor];
        if (shadowColor)
        {
            titleLabel.shadowColor = shadowColor;
        }
        
        [self addSubview:titleLabel];
    }
}

@end
